#!/bin/bash

# This script looks for the necessary BOOST sub-libraries.
# If found, it prints out two SHELL assignments (to be processed
# using the eval commmand) to BOOST_LIB_DIR & BOOST_LIBS.
# If not, it prints out a warning and returns with exit code 1.

SUBLIBS="thread filesystem system"
# The result will be either $BOOST_LDLIBS or $BOOST_SYMLINK_LIBS
BOOST_LDLIBS=
for sublib in $SUBLIBS
do
    BOOST_LDLIBS="$BOOST_LDLIBS -lboost_$sublib"
done
BOOST_SYMLINK_LDLIBS=
for sublib in $SUBLIBS
do
    BOOST_SYMLINK_LDLIBS="$BOOST_SYMLINK_LDLIBS -lboost_$sublib-symlink"
done

##################################################################
if [ "$#" != "1" -o -z "$1" ]
then
    echo "$0: ERROR: requires 1 non-empty arg (full path of BOOST_HDR_DIR)"
    exit 1
fi

BOOST_HDR_DIR="$1"

EXTLIBS=configuration/ExternalLibs

#############################################################################
# Find directory of this script (taken from link below)
# http://stackoverflow.com/questions/4774054/reliable-way-for-a-bash-script-to-get-the-full-path-to-itself

pushd . > /dev/null
SCRIPT_PATH="${BASH_SOURCE[0]}"
while [ -L "$SCRIPT_PATH" ]
do
  cd "`dirname "$SCRIPT_PATH"`"
  SCRIPT_PATH="$(readlink "`basename "$SCRIPT_PATH"`")"
done
cd "`dirname "$SCRIPT_PATH"`" > /dev/null
SCRIPT_DIR="`pwd`"
popd  > /dev/null


# Special handling if BOOST is not fully installed...
if [ -d "$BOOST_HDR_DIR/stage" -a -r "$BOOST_HDR_DIR/stage" -a -d "$BOOST_HDR_DIR/stage/lib" -a -r "$BOOST_HDR_DIR/stage/lib" ]
then
    BOOST_LIB_DIR="$BOOST_HDR_DIR/stage/lib"
else
    # CHEAP HACK: prefer lib64 over lib if possible -- GMP probably chose 64-bits
    BOOST_LIB_DIR_DIR="`dirname \"$BOOST_HDR_DIR\"`"
    if [ -d "$BOOST_LIB_DIR_DIR/lib64" ]
    then
	BOOST_LIB_DIR="$BOOST_LIB_DIR_DIR/lib64"
    else
	BOOST_LIB_DIR="$BOOST_LIB_DIR_DIR/lib"
    fi
fi

# Heuristic for recognizing standard installation of BOOST
BOOST_LIB_DIR_DIR=`dirname "$BOOST_LIB_DIR"`
if [ "$BOOST_LIB_DIR_DIR" = /usr -o "$BOOST_LIB_DIR_DIR" = /usr/local ]
then
    BOOST_IS_STANDARD=yes
fi

# Check that the SUBLIBS we need are present.
# Check first for multi-threaded versions...
MISSING_SUBLIBS_MT=
BOOST_LDLIBS_MT=
for sublib in $SUBLIBS
do
    /bin/ls "$BOOST_LIB_DIR"/libboost_$sublib-mt.* >/dev/null 2>&1
    if [ $? = 0 ]
    then
	BOOST_LDLIBS_MT="$BOOST_LDLIBS_MT -lboost_$sublib-mt"
    else
	MISSING_SUBLIBS_MT="$MISSING_SUBLIBS_MT libboost_$sublib-mt"
    fi
done

if [ -z "$MISSING_SUBLIBS_MT" ]
then
    if [ "$BOOST_IS_STANDARD" = yes ]
    then
	echo $BOOST_LDLIBS_MT
	exit 0
    fi
    ####  echo "BOOST_LIB_DIR=\"$BOOST_LIB_DIR\"; BOOST_LIBS=\"$BOOST_LIBS_MT\""
    for sublib in $SUBLIBS
    do
	BOOST_LIB_ORIG=`/bin/ls "$BOOST_LIB_DIR"/libboost_$sublib-mt.* | head -1`
	BOOST_LIB_EXTN=`$SCRIPT_DIR/extn.sh "$BOOST_LIB_ORIG"`
	/bin/ln -s "$BOOST_LIB_ORIG" $EXTLIBS/lib/libboost_$sublib-symlink.$BOOST_LIB_EXTN
    done
    echo $BOOST_SYMLINK_LDLIBS
    exit 0
fi

# Check now for single-threaded versions...
MISSING_SUBLIBS=
BOOST_LIBS=
for sublib in $SUBLIBS
do
    /bin/ls "$BOOST_LIB_DIR"/libboost_$sublib* >/dev/null 2>&1
    if [ $? = 0 ]
    then
	BOOST_LIBS="$BOOST_LIBS -lboost_$sublib"
    else
	MISSING_SUBLIBS="$MISSING_SUBLIBS libboost_$sublib"
    fi
done

if [ -z "$MISSING_SUBLIBS" ]
then
    if [ "$BOOST_IS_STANDARD" = yes ]
    then
	echo $BOOST_LDLIBS
	exit 0
    fi
    ####  echo "BOOST_LIB_DIR=\"$BOOST_LIB_DIR\"; BOOST_LIBS=\"$BOOST_LIBS\""
    for sublib in $SUBLIBS
    do
	BOOST_LIB_ORIG=`/bin/ls "$BOOST_LIB_DIR"/libboost_$sublib.* | head -1`
	BOOST_LIB_EXTN=`$SCRIPT_DIR/extn.sh "$BOOST_LIB_ORIG"`
	/bin/ln -s "$BOOST_LIB_ORIG" $EXTLIBS/lib/libboost_$sublib-symlink.$BOOST_LIB_EXTN
    done
    echo $BOOST_SYMLINK_LDLIBS
    exit 0
fi

# Neither single- nor multi-threaded libs had all we need, so give error.
if [ -z "$BOOST_LIBS_MT" ]
then
    echo "REASON: CoCoA-5 needs the following missing BOOST sublibraries: $MISSING_SUBLIBS"
else
    echo "REASON: CoCoA-5 needs the following missing BOOST sublibraries: $MISSING_SUBLIBS_MT"
fi
exit 1
